@@ -41,9 +41,11 @@ is" without express or implied warranty.
 #include "Init.h"
 #include "mipointer.h"
 #include "Args.h"
+#include "mipointrst.h"
 
 Window xnestDefaultWindows[MAXSCREENS];
 Window xnestScreenSaverWindows[MAXSCREENS];
+DevPrivateKeyRec xnestCursorScreenKeyRec;
 
 ScreenPtr
 xnestScreen(Window window)
@@ -124,6 +126,8 @@ static miPointerSpriteFuncRec xnestPointerSpriteFuncs =
     xnestUnrealizeCursor,
     xnestSetCursor,
     xnestMoveCursor,
+    xnestDeviceCursorInitialize,
+    xnestDeviceCursorCleanup
 };
 
 Bool
@@ -139,21 +143,24 @@ xnestOpenScreen(int index, ScreenPtr pScreen, int argc, char *argv[])
   XSizeHints sizeHints;
   VisualID defaultVisual;
   int rootDepth;
+  miPointerScreenPtr PointPriv;
 
-  if (!dixRequestPrivate(xnestWindowPrivateKey, sizeof(xnestPrivWin)))
-      return False;
-  if (!dixRequestPrivate(xnestGCPrivateKey, sizeof(xnestPrivGC)))
-    return False;
-  if (!dixRequestPrivate(xnestPixmapPrivateKey, sizeof (xnestPrivPixmap)))
-      return False;
+  if (!dixRegisterPrivateKey(&xnestWindowPrivateKeyRec, PRIVATE_WINDOW, sizeof(xnestPrivWin)))
+      return FALSE;
+  if (!dixRegisterPrivateKey(&xnestGCPrivateKeyRec, PRIVATE_GC, sizeof(xnestPrivGC)))
+    return FALSE;
+  if (!dixRegisterPrivateKey(&xnestPixmapPrivateKeyRec, PRIVATE_PIXMAP, sizeof (xnestPrivPixmap)))
+      return FALSE;
+  if (!dixRegisterPrivateKey(&xnestCursorScreenKeyRec, PRIVATE_SCREEN, 0))
+      return FALSE;
 
-  visuals = (VisualPtr)xalloc(xnestNumVisuals * sizeof(VisualRec));
+  visuals = (VisualPtr)malloc(xnestNumVisuals * sizeof(VisualRec));
   numVisuals = 0;
 
-  depths = (DepthPtr)xalloc(MAXDEPTH * sizeof(DepthRec));
+  depths = (DepthPtr)malloc(MAXDEPTH * sizeof(DepthRec));
   depths[0].depth = 1;
   depths[0].numVids = 0;
-  depths[0].vids = (VisualID *)xalloc(MAXVISUALSPERDEPTH * sizeof(VisualID));
+  depths[0].vids = (VisualID *)malloc(MAXVISUALSPERDEPTH * sizeof(VisualID));
   numDepths = 1;
 
   for (i = 0; i < xnestNumVisuals; i++) {
@@ -199,7 +206,7 @@ xnestOpenScreen(int index, ScreenPtr pScreen, int argc, char *argv[])
       depths[depthIndex].depth = xnestVisuals[i].depth;
       depths[depthIndex].numVids = 0;
       depths[depthIndex].vids = 
-	(VisualID *)xalloc(MAXVISUALSPERDEPTH * sizeof(VisualID));
+	(VisualID *)malloc(MAXVISUALSPERDEPTH * sizeof(VisualID));
       numDepths++;
     }
     if (depths[depthIndex].numVids >= MAXVISUALSPERDEPTH) {
@@ -211,7 +218,7 @@ xnestOpenScreen(int index, ScreenPtr pScreen, int argc, char *argv[])
     
     numVisuals++;
   }
-  visuals = (VisualPtr)xrealloc(visuals, numVisuals * sizeof(VisualRec));
+  visuals = (VisualPtr)realloc(visuals, numVisuals * sizeof(VisualRec));
 
   defaultVisual = visuals[xnestDefaultVisualIndex].vid;
   rootDepth = visuals[xnestDefaultVisualIndex].nplanes;
@@ -239,7 +246,6 @@ xnestOpenScreen(int index, ScreenPtr pScreen, int argc, char *argv[])
   pScreen->saveUnderSupport = NotUseful;
   pScreen->whitePixel = xnestWhitePixel;
   pScreen->blackPixel = xnestBlackPixel;
-  /* rgf */
   /* GCperDepth */
   /* PixmapPerDepth */
   pScreen->devPrivate = NULL;
@@ -256,7 +262,6 @@ xnestOpenScreen(int index, ScreenPtr pScreen, int argc, char *argv[])
   pScreen->SaveScreen = xnestSaveScreen;
   pScreen->GetImage = xnestGetImage;
   pScreen->GetSpans = xnestGetSpans;
-  pScreen->PointerNonInterestBox = NULL;
   pScreen->SourceValidate = NULL;
 
   /* Window Procedures */
@@ -305,8 +310,11 @@ xnestOpenScreen(int index, ScreenPtr pScreen, int argc, char *argv[])
   pScreen->blockData = NULL;
   pScreen->wakeupData = NULL;
 
-  miPointerInitialize (pScreen, &xnestPointerSpriteFuncs, 
-		       &xnestPointerCursorFuncs, True);
+  miDCInitialize(pScreen, &xnestPointerCursorFuncs); /* init SW rendering */
+  PointPriv = dixLookupPrivate(&pScreen->devPrivates, miPointerScreenKey);
+  xnestCursorFuncs.spriteFuncs = PointPriv->spriteFuncs;
+  dixSetPrivate(&pScreen->devPrivates, xnestCursorScreenKey, &xnestCursorFuncs);
+  PointPriv->spriteFuncs = &xnestPointerSpriteFuncs;
 
   pScreen->mmWidth = xnestWidth * DisplayWidthMM(xnestDisplay, 
 		       DefaultScreen(xnestDisplay)) / 
@@ -323,10 +331,8 @@ xnestOpenScreen(int index, ScreenPtr pScreen, int argc, char *argv[])
   if (!miScreenDevPrivateInit(pScreen, xnestWidth, NULL))
       return FALSE;
 
-#ifdef SHAPE
   /* overwrite miSetShape with our own */
   pScreen->SetShape = xnestSetShape;
-#endif /* SHAPE */
 
   /* devPrivates */
 
@@ -402,10 +408,10 @@ xnestCloseScreen(int index, ScreenPtr pScreen)
   int i;
   
   for (i = 0; i < pScreen->numDepths; i++)
-    xfree(pScreen->allowedDepths[i].vids);
-  xfree(pScreen->allowedDepths);
-  xfree(pScreen->visuals);
-  xfree(pScreen->devPrivate);
+    free(pScreen->allowedDepths[i].vids);
+  free(pScreen->allowedDepths);
+  free(pScreen->visuals);
+  free(pScreen->devPrivate);
 
   /*
     If xnestDoFullGeneration all x resources will be destroyed upon closing
