@@ -191,7 +191,7 @@ mieqEnqueue(DeviceIntPtr pDev, InternalEvent *e)
     if (evt->evlen < evlen)
     {
         evt->evlen = evlen;
-        evt->event = xrealloc(evt->event, evt->evlen);
+        evt->event = realloc(evt->event, evt->evlen);
         if (!evt->event)
         {
             ErrorF("[mi] Running out of memory. Tossing event.\n");
@@ -323,7 +323,8 @@ CopyGetMasterEvent(DeviceIntPtr sdev,
 
     CHECKEVENT(original);
 
-    if (!sdev->u.master)
+    /* ET_XQuartz has sdev == NULL */
+    if (!sdev || !sdev->u.master)
         return NULL;
 
     switch(original->any.type)
@@ -438,7 +439,7 @@ mieqProcessInputEvents(void)
 
         evlen   = e->events->evlen;
         if(evlen > event_size)
-            event = xrealloc(event, evlen);
+            event = realloc(event, evlen);
 
         if (!event)
             FatalError("[mi] No memory left for event processing.\n");
